"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateCode = void 0;
const path_1 = __importDefault(require("path"));
const Client = __importStar(require("./generators/client/generate"));
const Express = __importStar(require("./generators/express/generate"));
const ExpressWithDi = __importStar(require("./generators/express-with-di/generate"));
const Types = __importStar(require("./generators/types/generate"));
const MswHandlers = __importStar(require("./generators/msw-handlers/generate"));
const assertNever_1 = require("./helpers/assertNever");
const initUpper_1 = require("./helpers/initUpper");
const load_1 = require("./helpers/document-loader/load");
const writeFile_1 = require("./helpers/writeFile");
const makeIdentifier_1 = require("./templates/makeIdentifier");
const defaultOptions = {
    output: 'CLIENT'
};
const generateCode = async (inputOptions) => {
    const options = {
        ...defaultOptions,
        ...inputOptions
    };
    const openApiDocument = (await (0, load_1.load)(options.openApiDocumentFilenameOrUrl, options.logger));
    const serviceName = inputOptions.serviceName ?? (0, makeIdentifier_1.makeIdentifier)(openApiDocument.info.title);
    const filename = (() => {
        const outputTypeName = (0, initUpper_1.initUpper)(options.output.toLowerCase().replace('-', ' ')).replace(' ', '');
        const file = `${serviceName}${outputTypeName}.generated.ts`;
        if (!options.outputFilename) {
            return file;
        }
        if (path_1.default.extname(options.outputFilename)) {
            return options.outputFilename;
        }
        return path_1.default.join(options.outputFilename, file);
    })();
    const renderDeps = {
        ...options,
        openApiDocument,
        serviceName,
        filename
    };
    const method = (() => {
        switch (options.output) {
            case 'CLIENT':
            case 'client':
                return Client.generate;
            case 'SERVER':
            case 'express-di':
                return ExpressWithDi.generate;
            case 'express':
                return Express.generate;
            case 'TYPES':
            case 'types':
                return Types.generate;
            case 'msw-handlers':
                return MswHandlers.generate;
            default:
                return (0, assertNever_1.assertNever)(options.output);
        }
    })();
    await (0, writeFile_1.writeFile)({
        ...renderDeps,
        content: method(renderDeps)
    });
};
exports.generateCode = generateCode;
