"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
require("ts-array-extensions");
const cli_logging_1 = require("../../lib/cli-logging");
const methods_1 = require("./methods");
const TypeDefContext_1 = require("../../helpers/open-api/TypeDefContext");
const generate = ({ logger, openApiDocument, serviceName, nonRequiredType, readonlyDTOs, emptyType }) => {
    const log = logger?.create('Generating client');
    const context = new TypeDefContext_1.TypeDefContext();
    const methods = (0, methods_1.generateMethods)(openApiDocument, { nonRequiredType, readonlyDTOs, emptyType }, context, log);
    const code = `
    import qs from 'qs';

    ${context.generateCode()}

    type ResponseWithData<TStatus, TData> = {
      status: TStatus;
      body: TData;
    };
    
    type LegacyLogFn = (message: string, data: Record<string, unknown>) => void;

    type Logger = Pick<typeof console, 'info' | 'error'>;

    export interface ${serviceName}ClientOptions {
      fetch: typeof fetch;
      logger: Logger | null;
      requestOptions: RequestInit;
    } 

    export class ${serviceName}Client {
      public constructor(
        public readonly baseUrl: string,
        options?: Partial<${serviceName}ClientOptions> | LegacyLogFn
      ) {
        if (typeof options === 'function') {
          this.options = {
            fetch: global.fetch.bind(global),
            logger: {
              info: options,
              error: options
            },
            requestOptions: {}
          }
        } else {
          this.options = {
            fetch: global.fetch.bind(global),
            logger: console,
            requestOptions: {},
            ...options
          }
        };
      }

      private readonly options: ${serviceName}ClientOptions;
      
      ${methods.join('\n\n')}
    }
  `;
    log?.((0, cli_logging_1.success)());
    return code;
};
exports.generate = generate;
