"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromV2 = void 0;
const HelpersV2 = __importStar(require("../../../helpers/open-api/v2"));
const cli_logging_1 = require("../../../lib/cli-logging");
const templates_1 = require("../../../templates");
const fromV2 = (document, options, context, log) => {
    return HelpersV2.mapOperations(document).map(({ method, parameters, path, operationId, responses }) => {
        log?.((0, cli_logging_1.progress)(`Adding ${method} ${path}`));
        const jsonBody = () => {
            const bodyArg = parameters.filter(p => p.in === 'body').first();
            if (!bodyArg)
                return null;
            const { schema, required = false } = bodyArg;
            return {
                type: 'json',
                jsonType: HelpersV2.typeDefForSchema(schema, document, options, context),
                required
            };
        };
        const formDataBody = () => {
            const bodyArg = parameters.filter(p => p.in === 'formData');
            if (bodyArg.length === 0)
                return null;
            return {
                type: 'form',
                fields: bodyArg.map(p => ({
                    name: p.name,
                    required: p.required ?? false,
                    binary: p.type === 'file'
                })),
                required: true
            };
        };
        return (0, templates_1.clientMethodTemplate)({
            httpMethod: method,
            methodName: operationId,
            openApiPath: path,
            pathParams: parameters
                .filter(p => p.in === 'path')
                .map(p => ({
                name: p.name,
                type: HelpersV2.typeDefForSchema({ ...p, required: [] }, document, options, context)
            })),
            queryArrayFormat: parameters
                .filter(p => p.in === 'query')
                .some(p => p.collectionFormat === 'csv')
                ? 'comma'
                : 'repeat',
            queryParams: parameters
                .filter(p => p.in === 'query')
                .map(p => ({
                name: p.name,
                required: p.required ?? false,
                type: p.schema
                    ? HelpersV2.typeDefForSchema(
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                    p.schema, document, options, context)
                    : 'string'
            })),
            headerParams: parameters
                .filter(p => p.in === 'header')
                .map(p => ({
                name: p.name,
                required: p.required ?? false,
                type: p.schema
                    ? HelpersV2.typeDefForSchema(
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                    p.schema, document, options, context)
                    : 'string'
            })),
            body: jsonBody() ?? formDataBody(),
            responses: responses.map(({ statusCode, response }) => {
                if (response.schema?.type === 'file') {
                    return {
                        statusCode,
                        type: 'binary'
                    };
                }
                const { schema } = response;
                if (schema) {
                    return {
                        statusCode,
                        type: 'json',
                        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
                        jsonType: schema
                            ? HelpersV2.typeDefForSchema(schema, document, options, context)
                            : options.emptyType
                    };
                }
                return {
                    statusCode,
                    type: 'none'
                };
            })
        }, options);
    });
};
exports.fromV2 = fromV2;
