"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromV3 = void 0;
const HelpersV3 = __importStar(require("../../../helpers/open-api/v3"));
const cli_logging_1 = require("../../../lib/cli-logging");
const templates_1 = require("../../../templates");
const fromV3 = (document, options, context, log) => {
    return HelpersV3.mapOperations(document).map(({ method, parameters, path, operationId, responses, requestBody }) => {
        log?.((0, cli_logging_1.progress)(`Adding ${method} ${path}`));
        const body = (() => {
            if (!requestBody)
                return null;
            if ('application/json' in requestBody.content &&
                requestBody.content['application/json'] &&
                'schema' in requestBody.content['application/json']) {
                const { schema } = requestBody.content['application/json'];
                return {
                    type: 'json',
                    required: requestBody.required ?? false,
                    jsonType: schema
                        ? HelpersV3.typeDefForSchema(schema, document, options, context)
                        : options.emptyType
                };
            }
            if ('multipart/form-data' in requestBody.content &&
                requestBody.content['multipart/form-data'] &&
                'schema' in requestBody.content['multipart/form-data']) {
                const { schema } = requestBody.content['multipart/form-data'];
                if (!schema)
                    return null;
                const inlinedSchema = HelpersV3.isReferenceObject(schema)
                    ? HelpersV3.dereferenceSchemaObject(schema, document)
                    : schema;
                const fields = inlinedSchema.properties
                    ? Object.keys(inlinedSchema.properties).map(k => {
                        const property = inlinedSchema.properties?.[k];
                        const inlinedProperty = HelpersV3.isReferenceObject(property)
                            ? HelpersV3.dereferenceSchemaObject(property, document)
                            : property;
                        return {
                            name: k.toString(),
                            required: inlinedSchema.required?.includes(k.toString()) ?? false,
                            binary: inlinedProperty?.format === 'binary'
                        };
                    })
                    : [];
                return {
                    type: 'form',
                    required: requestBody.required,
                    fields
                };
            }
            return null;
        })();
        return (0, templates_1.clientMethodTemplate)({
            httpMethod: method,
            methodName: operationId,
            openApiPath: path,
            pathParams: parameters
                .filter(p => p.in === 'path')
                .map(p => ({
                name: p.name,
                type: p.schema
                    ? HelpersV3.typeDefForSchema(p.schema, document, options, context)
                    : 'string'
            })),
            queryArrayFormat: parameters
                .filter(p => p.in === 'query')
                .some(p => p.explode === false)
                ? 'comma'
                : 'repeat',
            queryParams: parameters
                .filter(p => p.in === 'query')
                .map(p => ({
                name: p.name,
                required: p.required ?? false,
                type: p.schema
                    ? HelpersV3.typeDefForSchema(p.schema, document, options, context)
                    : 'string'
            })),
            headerParams: parameters
                .filter(p => p.in === 'header')
                .map(p => ({
                name: p.name,
                required: p.required ?? false,
                type: p.schema
                    ? HelpersV3.typeDefForSchema(p.schema, document, options, context)
                    : 'string'
            })),
            body,
            responses: responses.map(({ statusCode, response }) => {
                if (!response.content ||
                    Object.keys(response.content).length === 0) {
                    return {
                        statusCode,
                        type: 'none'
                    };
                }
                if ('application/json' in response.content) {
                    const { schema } = response.content['application/json'];
                    return {
                        statusCode,
                        type: 'json',
                        jsonType: schema
                            ? HelpersV3.typeDefForSchema(schema, document, options, context)
                            : options.emptyType
                    };
                }
                if ('text/html' in response.content) {
                    return {
                        statusCode,
                        type: 'text'
                    };
                }
                return {
                    statusCode,
                    type: 'binary'
                };
            })
        }, options);
    });
};
exports.fromV3 = fromV3;
