"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromV2 = void 0;
const HelpersV2 = __importStar(require("../../../helpers/open-api/v2"));
require("ts-array-extensions");
const cli_logging_1 = require("../../../lib/cli-logging");
const templates_1 = require("../../../templates");
const controllersTemplate_1 = require("../templates/controllersTemplate");
const initLower_1 = require("../../../helpers/initLower");
const makeIdentifier_1 = require("../../../templates/makeIdentifier");
const fromV2 = (document, options, context, log) => {
    const handlers = HelpersV2.mapOperations(document).map(({ operationId, parameters, responses, path, method }) => {
        log?.((0, cli_logging_1.progress)(`Adding ${method} ${path}`));
        const responseBodyType = responses
            .compactMap(({ statusCode, response }) => {
            const { schema } = response;
            if (!schema) {
                return undefined;
            }
            const name = (0, templates_1.responseBodyTypeNameTemplate)(operationId, statusCode);
            context.emitType(name, HelpersV2.typeDefForSchema(schema, document, { ...options, nonRequiredType: 'optional' }, context));
            return name;
        })
            .join(' | ') || undefined;
        const statusCodeType = responses
            .map(({ statusCode }) => statusCode === 'default' ? 'number' : statusCode)
            .join(' | ');
        const nameAndSchemaForParams = (type) => {
            const params = parameters.filter(p => p.in === type);
            if (params.none()) {
                return [undefined, undefined];
            }
            if (type === 'body') {
                const schemas = params.map(p => p.schema);
                const typeName = (0, templates_1.parametersTypeNameTemplate)(operationId, 'body');
                log?.((0, cli_logging_1.progress)(`adding ${typeName}`));
                context.emitType(typeName, HelpersV2.typeDefForSchema(schemas, document, options, context));
                return [
                    typeName,
                    HelpersV2.exportAsJsonSchema(schemas, document)
                ];
            }
            const typeName = (0, templates_1.parametersTypeNameTemplate)(operationId, type);
            const schema = {
                type: 'object',
                properties: {},
                required: []
            };
            params.forEach(v => {
                const { in: _, name, required, ...rest } = v;
                if (rest.type === 'file') {
                    return;
                }
                const propertyType = (() => {
                    if (rest.type === 'array') {
                        return { anyOf: [rest, rest.items] };
                    }
                    return rest;
                })();
                schema.properties[name] = propertyType;
                if (required) {
                    schema.required = [...schema.required, name];
                }
            });
            context.emitType(typeName, HelpersV2.typeDefForSchema(schema, document, options, context));
            return [
                typeName,
                HelpersV2.exportAsJsonSchema(schema, document)
            ];
        };
        const [bodyParamType, bodySchema] = nameAndSchemaForParams('body');
        const [queryParamsType, querySchema] = nameAndSchemaForParams('query');
        const [pathParamsType, pathSchema] = nameAndSchemaForParams('path');
        const operationIdAsIdentifier = (0, makeIdentifier_1.makeIdentifier)(operationId);
        const requestTypeName = `${operationIdAsIdentifier}Request`;
        const responseTypeName = `${operationIdAsIdentifier}Response`;
        const controllerTypeName = `${operationIdAsIdentifier}Controller`;
        const controllerMethodName = (0, initLower_1.initLower)(operationIdAsIdentifier);
        const expressPath = path.replace(/\{(?:.*?)\}/g, x => `:${x.substring(1, x.length - 1)}`);
        log?.((0, cli_logging_1.progress)(operationId));
        return {
            httpMethod: method,
            expressPath,
            requestTypeName,
            responseTypeName,
            controllerTypeName,
            controllerMethodName,
            pathParamsType,
            responseBodyType: responseBodyType ?? options.emptyType,
            bodyParamType: bodyParamType ?? options.emptyType,
            queryParamsType,
            statusCodeType,
            bodySchema,
            querySchema,
            pathSchema
        };
    });
    return (0, controllersTemplate_1.controllersTemplate)(handlers);
};
exports.fromV2 = fromV2;
