"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromV3 = void 0;
const HelpersV3 = __importStar(require("../../../helpers/open-api/v3"));
require("ts-array-extensions");
const cli_logging_1 = require("../../../lib/cli-logging");
const templates_1 = require("../../../templates");
const initLower_1 = require("../../../helpers/initLower");
const makeIdentifier_1 = require("../../../templates/makeIdentifier");
const handlersTemplate_1 = require("../templates/handlersTemplate");
const convertFormDataToSchemaObject = (formData) => {
    if (!formData?.schema)
        return undefined;
    return {
        type: 'object',
        ...formData.schema
    };
};
const fromV3 = (document, options, context, log) => {
    const handlers = HelpersV3.mapOperations(document).map(({ path, operationId, parameters, responses, requestBody, method }) => {
        const responseBodyType = responses
            .compactMap(({ statusCode, response }) => {
            if (!response.content ||
                !('application/json' in response.content)) {
                return undefined;
            }
            const { schema } = response.content['application/json'];
            const name = (0, templates_1.responseBodyTypeNameTemplate)(operationId, statusCode);
            if (schema) {
                context.emitType(name, HelpersV3.typeDefForSchema(schema, document, options, context));
            }
            return name;
        })
            .join(' | ') || undefined;
        const statusCodeType = responses
            .map(({ statusCode }) => statusCode === 'default' ? 'number' : statusCode)
            .join(' | ');
        const [bodyParamType, bodySchema] = (() => {
            if (!requestBody) {
                return [undefined, undefined];
            }
            const structuredContent = [
                requestBody.content['application/json']?.schema,
                convertFormDataToSchemaObject(requestBody.content['multipart/form-data'])
            ].compact();
            if (structuredContent.length === 0) {
                return [undefined, undefined];
            }
            const schema = structuredContent.length > 1
                ? {
                    anyOf: structuredContent
                }
                : structuredContent[0];
            const typeName = (0, templates_1.parametersTypeNameTemplate)(operationId, 'body');
            context.emitType(typeName, HelpersV3.typeDefForSchema(schema, document, options, context));
            return [typeName, HelpersV3.exportAsJsonSchema(schema, document)];
        })();
        const nameAndSchemaForParams = (type) => {
            const params = parameters.filter(p => p.in === type);
            if (params.none()) {
                return [undefined, undefined];
            }
            const typeName = (0, templates_1.parametersTypeNameTemplate)(operationId, type);
            const schema = {
                type: 'object',
                properties: {},
                required: []
            };
            params.forEach(v => {
                const { in: _, name, required, ...rest } = v;
                const propertyType = (() => {
                    if (!rest.schema) {
                        return { type: 'string' };
                    }
                    if ('$ref' in rest.schema) {
                        return rest.schema;
                    }
                    if (rest.schema.type === 'array') {
                        return {
                            anyOf: [rest.schema, rest.schema.items]
                        };
                    }
                    return rest.schema;
                })();
                schema.properties[name] = propertyType;
                if (required) {
                    schema.required = [...schema.required, name];
                }
            });
            context.emitType(typeName, HelpersV3.typeDefForSchema(schema, document, options, context));
            return [
                typeName,
                HelpersV3.exportAsJsonSchema(schema, document)
            ];
        };
        const [queryParamsType, querySchema] = nameAndSchemaForParams('query');
        const [pathParamsType, pathSchema] = nameAndSchemaForParams('path');
        const operationIdAsIdentifier = (0, makeIdentifier_1.makeIdentifier)(operationId);
        const requestTypeName = `${operationIdAsIdentifier}Request`;
        const responseTypeName = `${operationIdAsIdentifier}Response`;
        const handlerTypeName = `${operationIdAsIdentifier}Handler`;
        const handlerInstanceName = (0, initLower_1.initLower)(operationIdAsIdentifier);
        const expressPath = path.replace(/\{(?:.*?)\}/g, x => `:${x.substring(1, x.length - 1)}`);
        log?.((0, cli_logging_1.progress)(operationId));
        return {
            httpMethod: method,
            expressPath,
            requestTypeName,
            responseTypeName,
            handlerTypeName,
            handlerInstanceName,
            operationId,
            pathParamsType,
            pathSchema,
            responseBodyType: responseBodyType ?? options.emptyType,
            bodyParamType: bodyParamType ?? options.emptyType,
            bodySchema,
            queryParamsType,
            querySchema,
            statusCodeType
        };
    });
    return (0, handlersTemplate_1.handlersTemplate)(handlers);
};
exports.fromV3 = fromV3;
