"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromV2 = void 0;
const HelpersV2 = __importStar(require("../../../helpers/open-api/v2"));
const cli_logging_1 = require("../../../lib/cli-logging");
const makeIdentifier_1 = require("../../../templates/makeIdentifier");
const fromV2 = (document, options, context, log) => {
    return HelpersV2.mapOperations(document).map(({ method, parameters, operationId, path, responses }) => {
        log?.((0, cli_logging_1.progress)(`Adding from ${method} ${path}`));
        const operationTypePrefix = (0, makeIdentifier_1.makeIdentifier)(operationId);
        const parameterTypes = [
            'header',
            'path',
            'query',
            'body'
        ];
        const [headerType, pathType, queryType, requestBodyType] = parameterTypes.map(paramType => {
            const paramsOfType = parameters.filter(p => p.in === paramType);
            return HelpersV2.typeDefForSchema({
                type: 'object',
                properties: Object.fromEntries(paramsOfType.map(p => [p.name, p.schema ?? { type: 'string' }])),
                required: paramsOfType.filter(p => p.required).map(p => p.name)
            }, document, options, context);
        });
        context.emitType(`${operationTypePrefix}RequestQuery`, queryType);
        context.emitType(`${operationTypePrefix}RequestHeaders`, headerType);
        const requestParamsTypeName = `${operationTypePrefix}RequestParams`;
        context.emitType(requestParamsTypeName, pathType);
        const requestBodyTypeName = `${operationTypePrefix}RequestBody`;
        context.emitType(requestBodyTypeName, requestBodyType);
        const statusCodeType = responses
            .map(({ statusCode }) => statusCode === 'default' ? 'number' : statusCode)
            .distinct()
            .join(' | ');
        context.emitType(`${operationTypePrefix}ResponseStatus`, statusCodeType);
        const responseBodyType = responses
            .compactMap(({ response }) => {
            if (response.schema?.type !== 'file') {
                const { schema } = response;
                if (schema) {
                    return HelpersV2.typeDefForSchema(schema, document, options, context);
                }
            }
            return undefined;
        })
            .distinct()
            .join(' | ') || options.emptyType;
        const responseBodyTypeName = `${operationTypePrefix}ResponseBody`;
        context.emitType(responseBodyTypeName, responseBodyType);
        const resolverTypeName = `${operationTypePrefix}Resolver`;
        context.emitType(resolverTypeName, `msw.HttpResponseResolver<${requestParamsTypeName}, ${requestBodyTypeName}, ${responseBodyTypeName}>`);
        const expressPath = path.replace(/\{(?:.*?)\}/g, x => `:${x.substring(1, x.length - 1)}`);
        return `
        export function handle${operationTypePrefix}(
          resolver: ${resolverTypeName}
        ) {
          return msw.http.${method}('${expressPath}', resolver);
        }
      `;
    });
};
exports.fromV2 = fromV2;
