"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromV3 = void 0;
const HelpersV3 = __importStar(require("../../../helpers/open-api/v3"));
const cli_logging_1 = require("../../../lib/cli-logging");
const makeIdentifier_1 = require("../../../templates/makeIdentifier");
const fromV3 = (document, options, context, log) => {
    return HelpersV3.mapOperations(document).map(({ method, operationId, parameters, path, responses, requestBody }) => {
        log?.((0, cli_logging_1.progress)(`Adding from ${method} ${path}`));
        const operationTypePrefix = (0, makeIdentifier_1.makeIdentifier)(operationId);
        const requestBodyType = (() => {
            if (!requestBody)
                return options.emptyType;
            if ('application/json' in requestBody.content &&
                requestBody.content['application/json'] &&
                'schema' in requestBody.content['application/json']) {
                const { schema } = requestBody.content['application/json'];
                if (schema) {
                    return [
                        HelpersV3.typeDefForSchema(schema, document, options, context),
                        !requestBody.required ? 'undefined' : null
                    ]
                        .compact()
                        .distinct()
                        .join('|');
                }
            }
            if ('multipart/form-data' in requestBody.content &&
                requestBody.content['multipart/form-data'] &&
                'schema' in requestBody.content['multipart/form-data']) {
                const { schema } = requestBody.content['multipart/form-data'];
                if (schema) {
                    return [
                        HelpersV3.typeDefForSchema(schema, document, options, context),
                        !requestBody.required ? 'undefined' : null
                    ]
                        .compact()
                        .distinct()
                        .join('|');
                }
            }
            return options.emptyType;
        })();
        const parameterTypes = [
            'header',
            'path',
            'query'
        ];
        const [headerType, pathType, queryType] = parameterTypes.map(paramType => {
            const paramsOfType = parameters.filter(p => p.in === paramType);
            return HelpersV3.typeDefForSchema({
                type: 'object',
                properties: Object.fromEntries(paramsOfType.map(p => {
                    // for msw, params can only be string or string[]
                    const inSchema = p.schema && '$ref' in p.schema
                        ? HelpersV3.dereferenceSchemaObject(p.schema, document)
                        : p.schema;
                    const schema = inSchema?.type === 'array'
                        ? { type: 'array', items: { type: 'string' } }
                        : { type: 'string' };
                    return [p.name, schema];
                })),
                required: paramsOfType.filter(p => p.required).map(p => p.name)
            }, document, options, context);
        });
        context.emitType(`${operationTypePrefix}RequestQuery`, queryType);
        context.emitType(`${operationTypePrefix}RequestHeaders`, headerType);
        const requestParamsTypeName = `${operationTypePrefix}RequestParams`;
        context.emitType(requestParamsTypeName, pathType);
        const requestBodyTypeName = `${operationTypePrefix}RequestBody`;
        context.emitType(requestBodyTypeName, requestBodyType);
        const statusCodeType = responses
            .map(({ statusCode }) => statusCode === 'default' ? 'number' : statusCode)
            .distinct()
            .join(' | ');
        context.emitType(`${operationTypePrefix}ResponseStatus`, statusCodeType);
        const responseBodyType = responses
            .map(({ response }) => {
            if (!response.content || Object.keys(response.content).length === 0) {
                return options.emptyType;
            }
            if ('application/json' in response.content) {
                const { schema } = response.content['application/json'];
                return schema
                    ? HelpersV3.typeDefForSchema(schema, document, options, context)
                    : options.emptyType;
            }
            if ('text/html' in response.content) {
                return 'string';
            }
            return 'Buffer';
        })
            .distinct()
            .join(' | ');
        const responseBodyTypeName = `${operationTypePrefix}ResponseBody`;
        context.emitType(responseBodyTypeName, responseBodyType);
        const resolverTypeName = `${operationTypePrefix}Resolver`;
        context.emitType(resolverTypeName, `msw.HttpResponseResolver<${requestParamsTypeName}, ${requestBodyTypeName}, ${responseBodyTypeName}>`);
        const expressPath = path.replace(/\{(?:.*?)\}/g, x => `:${x.substring(1, x.length - 1)}`);
        return `
        export function handle${operationTypePrefix}(
          resolver: ${resolverTypeName}
        ) {
          return msw.http.${method}('${expressPath}', resolver);
        }
      `;
    });
};
exports.fromV3 = fromV3;
