"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportAsJsonSchema = void 0;
const json_pointer_1 = __importDefault(require("json-pointer"));
const JsonWalker_1 = require("../../JsonWalker");
const isReferenceObject_1 = require("./isReferenceObject");
const mapProperties = (properties) => {
    const r = {};
    Object.keys(properties).forEach(key => {
        r[key] = mapSchema(properties[key]);
    });
    return r;
};
const mapSchema = (schemas) => {
    /**
     * This currently does nothing, but is a placeholder for removing
     * unsupported features from Open Api
     */
    const convert = (schema) => {
        const { properties, ...rest } = schema;
        return {
            ...rest,
            ...(properties ? { properties: mapProperties(properties) } : undefined)
        };
    };
    const converted = (() => {
        if (Array.isArray(schemas)) {
            if (schemas.length === 1) {
                return convert(schemas[0]);
            }
            return { anyOf: schemas.map(convert) };
        }
        return convert(schemas);
    })();
    return converted;
};
const bundleReferences = (schema, document) => {
    const visited = [];
    const walker = (obj, ptr) => {
        if (!(0, isReferenceObject_1.isReferenceObject)(obj) || visited.includes(ptr)) {
            return;
        }
        visited.push(ptr);
        const pointer = obj.$ref.split('#')[1];
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
        const target = mapSchema(json_pointer_1.default.get(document, pointer));
        json_pointer_1.default.set(schema, pointer, target);
        obj.$ref = `#${pointer}`;
        JsonWalker_1.JsonWalker.walk(schema, walker, pointer);
    };
    JsonWalker_1.JsonWalker.walk(schema, walker);
};
const exportAsJsonSchema = (schemas, document) => {
    const converted = mapSchema(schemas);
    bundleReferences(converted, document);
    return converted;
};
exports.exportAsJsonSchema = exportAsJsonSchema;
