"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapOperations = void 0;
const dereferenceParameter_1 = require("./dereferenceParameter");
const dereferenceResponseObject_1 = require("./dereferenceResponseObject");
const inferOperationId_1 = require("../../inferOperationId");
const isReferenceObject_1 = require("./isReferenceObject");
const methodsOfPathItemObject_1 = require("./methodsOfPathItemObject");
const mapOperations = (document) => {
    return Object.keys(document.paths).flatMap(path => {
        if (typeof path !== 'string')
            return [];
        const pathObject = document.paths[path];
        const pathParameters = pathObject.parameters ?? [];
        return (0, methodsOfPathItemObject_1.methodsOfPathItemObject)(pathObject).map(([method, operationObject]) => {
            const { operationId = (0, inferOperationId_1.inferOperationId)(method, path), parameters = [], responses } = operationObject;
            const inlinedParameters = pathParameters
                .concat(parameters)
                .map(p => (0, isReferenceObject_1.isReferenceObject)(p) ? (0, dereferenceParameter_1.dereferenceParameter)(p, document) : p);
            const inlinedResponses = Object.keys(responses).compactMap(statusCode => {
                const response = responses[statusCode];
                if (!response)
                    return undefined;
                return {
                    statusCode: statusCode.toString(),
                    response: (0, isReferenceObject_1.isReferenceObject)(response)
                        ? (0, dereferenceResponseObject_1.dereferenceResponseObject)(response, document)
                        : response
                };
            });
            return {
                path,
                method,
                operationId,
                operation: operationObject,
                parameters: inlinedParameters,
                responses: inlinedResponses
            };
        });
    });
};
exports.mapOperations = mapOperations;
