"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.typeDefForSchema = void 0;
const typeDefForObject_1 = require("./typeDefForObject");
const typeDefForReference_1 = require("./typeDefForReference");
const typeDefForEnum_1 = require("./typeDefForEnum");
const makeIdentifier_1 = require("../../../templates/makeIdentifier");
const typeDefForSchema = (schemas, document, options, context) => {
    const gen = (schema) => {
        if (typeof schema === 'boolean') {
            throw new Error(`Schema nodes must be an object.`);
        }
        if ('$ref' in schema) {
            return (0, typeDefForReference_1.typeDefForReference)(schema, document, options, context, exports.typeDefForSchema);
        }
        const { enum: enumProp, allOf, title } = schema;
        const type = title ? (0, makeIdentifier_1.makeIdentifier)(title) : undefined;
        if (type) {
            if (!context.shouldEmitType(type)) {
                return type;
            }
            context.willEmitType(type);
        }
        const code = (() => {
            if (enumProp) {
                return (0, typeDefForEnum_1.typeDefForEnum)(enumProp);
            }
            if (schema.type === 'object') {
                const def = (0, typeDefForObject_1.typeDefForObject)(schema, document, exports.typeDefForSchema, options, context);
                if (allOf) {
                    return allOf
                        .map(m => (0, exports.typeDefForSchema)(m, document, options, context))
                        .concat(def)
                        .join(' & ');
                }
                return def;
            }
            if (allOf) {
                return allOf
                    .map(m => (0, exports.typeDefForSchema)(m, document, options, context))
                    .join(' & ');
            }
            if (schema.type === 'boolean' ||
                schema.type === 'null' ||
                schema.type === 'number' ||
                schema.type === 'string') {
                return schema.type;
            }
            if (schema.type === 'integer') {
                return 'number';
            }
            if (schema.type === 'array') {
                const { items } = schema;
                if (!items) {
                    throw new Error(`Array nodes must have an items property.`);
                }
                if (Array.isArray(items)) {
                    throw new Error(`Array items property must be singular.`);
                }
                const arrayType = options.readonlyDTOs ? 'ReadonlyArray' : 'Array';
                return `${arrayType}<${(0, exports.typeDefForSchema)(items, document, options, context)}>`;
            }
            return options.emptyType;
        })();
        if (type) {
            context.emitType(type, code);
            return type;
        }
        return code;
    };
    if (Array.isArray(schemas)) {
        return schemas.map(gen).join(' | ');
    }
    return gen(schemas);
};
exports.typeDefForSchema = typeDefForSchema;
