"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportAsJsonSchema = void 0;
const json_pointer_1 = __importDefault(require("json-pointer"));
const JsonWalker_1 = require("../../JsonWalker");
const isReferenceObject_1 = require("./isReferenceObject");
const mapProperties = (properties) => {
    const r = {};
    Object.keys(properties).forEach(key => {
        const property = properties[key];
        if ('format' in property && property.format === 'binary') {
            return;
        }
        r[key] = mapSchema(properties[key]);
    });
    return r;
};
const mapSchema = (schema) => {
    return '$ref' in schema
        ? { $ref: schema.$ref }
        : {
            ...schema,
            ...('properties' in schema && schema.properties
                ? { properties: mapProperties(schema.properties) }
                : undefined),
            ...(schema.allOf
                ? {
                    allOf: schema.allOf.map(mapSchema)
                }
                : undefined),
            ...(schema.anyOf
                ? {
                    anyOf: schema.anyOf.map(mapSchema)
                }
                : undefined),
            ...(schema.oneOf
                ? {
                    oneOf: schema.oneOf.map(mapSchema)
                }
                : undefined),
            ...('items' in schema ? { items: mapSchema(schema.items) } : undefined)
        };
};
const bundleReferences = (schema, document) => {
    const visited = [];
    const walker = (obj, ptr) => {
        if (!(0, isReferenceObject_1.isReferenceObject)(obj) || visited.includes(ptr)) {
            return;
        }
        visited.push(ptr);
        const docPointer = obj.$ref.split('#')[1];
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
        const target = mapSchema(json_pointer_1.default.get(document, docPointer));
        const convertedPointer = `/definitions/${docPointer.substring(docPointer.lastIndexOf('/') + 1)}`;
        json_pointer_1.default.set(schema, convertedPointer, target);
        obj.$ref = `#${convertedPointer}`;
        JsonWalker_1.JsonWalker.walk(target, walker, convertedPointer);
    };
    JsonWalker_1.JsonWalker.walk(schema, walker);
};
const exportAsJsonSchema = (schema, document) => {
    const converted = mapSchema(schema);
    bundleReferences(converted, document);
    return converted;
};
exports.exportAsJsonSchema = exportAsJsonSchema;
