"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapOperations = void 0;
const inferOperationId_1 = require("../../inferOperationId");
const dereferenceParameter_1 = require("./dereferenceParameter");
const dereferenceRequestBodyObject_1 = require("./dereferenceRequestBodyObject");
const dereferenceResponseObject_1 = require("./dereferenceResponseObject");
const getSecurityParams_1 = require("./getSecurityParams");
const isReferenceObject_1 = require("./isReferenceObject");
const methodsOfPathItemObject_1 = require("./methodsOfPathItemObject");
const mapOperations = (document) => {
    return Object.keys(document.paths).flatMap(path => {
        if (typeof path !== 'string')
            return [];
        const pathObject = document.paths[path];
        const pathParameters = pathObject.parameters ?? [];
        const globalSecurityParameters = (0, getSecurityParams_1.getSecurityParams)(document, document);
        return (0, methodsOfPathItemObject_1.methodsOfPathItemObject)(pathObject).map(([method, operationObject]) => {
            const { operationId = (0, inferOperationId_1.inferOperationId)(method, path), parameters = [], responses, requestBody } = operationObject;
            const inlinedParameters = pathParameters
                .concat(parameters)
                .map(p => (0, isReferenceObject_1.isReferenceObject)(p) ? (0, dereferenceParameter_1.dereferenceParameter)(p, document) : p)
                .concat(globalSecurityParameters)
                .concat((0, getSecurityParams_1.getSecurityParams)(document, operationObject))
                .groupBy(p => p.name)
                .compactMap(g => g.values.first());
            const inlinedRequestBody = requestBody && (0, isReferenceObject_1.isReferenceObject)(requestBody)
                ? (0, dereferenceRequestBodyObject_1.dereferenceRequestBodyObject)(requestBody, document)
                : requestBody;
            const inlinedResponses = Object.keys(responses).compactMap(statusCode => {
                const response = responses?.[statusCode];
                if (!response)
                    return undefined;
                return {
                    statusCode: statusCode.toString(),
                    response: (0, isReferenceObject_1.isReferenceObject)(response)
                        ? (0, dereferenceResponseObject_1.dereferenceResponseObject)(response, document)
                        : response
                };
            });
            return {
                path,
                method,
                operationId,
                operation: operationObject,
                parameters: inlinedParameters,
                requestBody: inlinedRequestBody,
                responses: inlinedResponses
            };
        });
    });
};
exports.mapOperations = mapOperations;
