"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.typeDefForSchema = void 0;
const typeDefForObject_1 = require("./typeDefForObject");
const typeDefForReference_1 = require("./typeDefForReference");
const typeDefForEnum_1 = require("./typeDefForEnum");
const isObjectSchema_1 = require("./isObjectSchema");
const makeIdentifier_1 = require("../../../templates/makeIdentifier");
const typeDefForSchema = (schema, document, options, context) => {
    if ('$ref' in schema) {
        return (0, typeDefForReference_1.typeDefForReference)(schema, document, options, context, exports.typeDefForSchema);
    }
    const { binaryType = 'Buffer' } = options;
    const { enum: enumProp, allOf, oneOf, anyOf, title } = schema;
    const type = title ? (0, makeIdentifier_1.makeIdentifier)(title) : undefined;
    if (type) {
        if (!context.shouldEmitType(type)) {
            return type;
        }
        context.willEmitType(type);
    }
    const preliminaryType = (() => {
        if (enumProp) {
            return (0, typeDefForEnum_1.typeDefForEnum)(enumProp);
        }
        const base = (0, isObjectSchema_1.isObjectSchema)(schema)
            ? (0, typeDefForObject_1.typeDefForObject)(schema, document, exports.typeDefForSchema, options, context)
            : null;
        if (allOf) {
            return allOf
                .map(m => (0, exports.typeDefForSchema)(m, document, options, context))
                .concat(base ? [base] : [])
                .distinct()
                .join(' & ');
        }
        const unionOf = (oneOf ?? []).concat(anyOf ?? []);
        if (unionOf.length > 0) {
            const union = unionOf
                .map(m => (0, exports.typeDefForSchema)(m, document, options, context))
                .distinct()
                .join(' | ');
            return base ? `${base} & (${union})` : union;
        }
        if (!schema.type) {
            // default to object
            return base ?? options.emptyType;
        }
        if (schema.type === 'object') {
            return base ?? options.emptyType;
        }
        if (schema.type === 'string') {
            if (schema.format === 'binary') {
                return binaryType;
            }
            return 'string';
        }
        if (schema.type === 'boolean' || schema.type === 'number') {
            return schema.type;
        }
        if (schema.type === 'integer') {
            return 'number';
        }
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (schema.type === 'array') {
            const { items } = schema;
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            if (!items) {
                throw new Error(`Array nodes must have an items property.`);
            }
            if (Array.isArray(items)) {
                throw new Error(`Array items property must be singular.`);
            }
            const arrayType = options.readonlyDTOs ? 'ReadonlyArray' : 'Array';
            return `${arrayType}<${(0, exports.typeDefForSchema)(items, document, options, context)}>`;
        }
        return options.emptyType;
    })();
    const code = 'nullable' in schema && schema.nullable
        ? `${preliminaryType} | null`
        : preliminaryType;
    if (type) {
        context.emitType(type, code);
        return type;
    }
    return code;
};
exports.typeDefForSchema = typeDefForSchema;
