"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeFile = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const prettier_1 = __importDefault(require("prettier"));
const cli_logging_1 = require("../lib/cli-logging");
const prettierStyle = {
    arrowParens: 'avoid',
    semi: true,
    trailingComma: 'none',
    singleQuote: true,
    printWidth: 80
};
const writeFile = async ({ content, logger, filename }) => {
    const log = logger?.create('Writing file');
    log?.((0, cli_logging_1.progress)('Creating output directory'));
    fs_1.default.mkdirSync(path_1.default.dirname(filename), { recursive: true });
    log?.((0, cli_logging_1.progress)('Formatting content'));
    const extension = path_1.default.extname(filename);
    const parser = (() => {
        switch (extension) {
            case '.ts':
                return 'typescript';
            case '.json':
                return 'json';
            default:
                return undefined;
        }
    })();
    const formattedContent = await (async () => {
        try {
            return await prettier_1.default.format(content, {
                ...prettierStyle,
                parser
            });
        }
        catch (e) {
            if (e instanceof Error) {
                log?.((0, cli_logging_1.warn)(e.message));
            }
            return content;
        }
    })();
    fs_1.default.writeFileSync(filename, formattedContent);
    log?.((0, cli_logging_1.success)());
};
exports.writeFile = writeFile;
