"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeIdentifier = exports.IdentifierFormat = void 0;
const initCap_1 = require("../helpers/initCap");
class IdentifierFormat {
    joinWords;
    constructor(joinWords) {
        this.joinWords = joinWords;
    }
    static pascalCase = new IdentifierFormat(words => words.map(initCap_1.initCap).join(''));
    static camelCase = new IdentifierFormat(words => words.map((w, i) => (i !== 0 ? (0, initCap_1.initCap)(w) : w.toLowerCase())).join(''));
}
exports.IdentifierFormat = IdentifierFormat;
function getCharClass(char) {
    if (/[A-Z]/.test(char))
        return 'UPPER';
    if (/[a-z]/.test(char))
        return 'LOWER';
    if (/[0-9]/.test(char))
        return 'NUMBER';
    return 'OTHER';
}
function* getWords(input) {
    let currentWord = [];
    for (const char of input) {
        const charClass = getCharClass(char);
        const lastChar = currentWord.last();
        // start of new word
        if (!lastChar) {
            if (charClass !== 'OTHER') {
                currentWord.push(char);
            }
            continue;
        }
        // non-upper to upper splits
        if (charClass === 'UPPER' && getCharClass(lastChar) !== 'UPPER') {
            yield currentWord.join('');
            currentWord = [char];
            continue;
        }
        // some other char always splits
        if (charClass === 'OTHER' && currentWord.any()) {
            yield currentWord.join('');
            currentWord = [];
            continue;
        }
        currentWord.push(char);
    }
    if (currentWord.any()) {
        yield currentWord.join('');
    }
}
const makeIdentifier = (source, format = IdentifierFormat.pascalCase) => {
    return format.joinWords([...getWords(source)]);
};
exports.makeIdentifier = makeIdentifier;
