"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const makeIdentifier_1 = require("./makeIdentifier");
require("ts-array-extensions");
describe('typeName', () => {
    it('underscored', () => {
        expect((0, makeIdentifier_1.makeIdentifier)('WIDGET_BUILDER')).toEqual('WidgetBuilder');
    });
    it('snaked', () => {
        expect((0, makeIdentifier_1.makeIdentifier)('widget-builder')).toEqual('WidgetBuilder');
    });
    it('camel', () => {
        expect((0, makeIdentifier_1.makeIdentifier)('widgetBuilder')).toEqual('WidgetBuilder');
    });
    it('pascal', () => {
        expect((0, makeIdentifier_1.makeIdentifier)('WidgetBuilder')).toEqual('WidgetBuilder');
    });
    it('mixed', () => {
        expect((0, makeIdentifier_1.makeIdentifier)('default_widget-builder')).toEqual('DefaultWidgetBuilder');
    });
    it('removes illegal chargs', () => {
        expect((0, makeIdentifier_1.makeIdentifier)('default!#widget builder$')).toEqual('DefaultWidgetBuilder');
    });
});
describe('formally serviceName', () => {
    it('removes spaces', () => {
        const result = (0, makeIdentifier_1.makeIdentifier)('A Wonderful Service');
        expect(result.includes(' ')).toBe(false);
    });
    it('removes full stops', () => {
        const result = (0, makeIdentifier_1.makeIdentifier)('funkyname.com service');
        expect(result.includes('.')).toBe(false);
    });
    it('produces the expected output', () => {
        const result = (0, makeIdentifier_1.makeIdentifier)(`funkyname.com's dollar$ service by @laurence79 [dev]`);
        expect(result).toEqual('FunkynameComSDollarServiceByLaurence79Dev');
    });
});
describe('Camel case', () => {
    it('lowercase the first word', () => {
        const result = (0, makeIdentifier_1.makeIdentifier)('X-Request-Id', makeIdentifier_1.IdentifierFormat.camelCase);
        expect(result).toEqual('xRequestId');
    });
});
